/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "project_info.h"

// The project information dialog initializes the object, set their properties,
// creates the dialog & shows it. It receives the project properties object.
ProjectInfoDialog::ProjectInfoDialog (ProjectSettings *project, ProjectTemplate *ptemplate)
{
  // Initialize the objects
  Glib::ustring filenames = project->get_source_files() + project->get_header_files();
  initialize_objects();

  ModelColumnsInfo columns;
  Gtk::TreeModel::Row row;
  Glib::RefPtr<Gtk::ListStore> model = Gtk::ListStore::create(columns);
  
  row = *(model->append());
  row[columns.col_statistic] = "Project Name";
  row[columns.col_value] = project->get_name();
  row = *(model->append());
  row[columns.col_statistic] = "Current Version";
  row[columns.col_value] = project->get_version();
  row = *(model->append());
  row[columns.col_statistic] = "Language";
  row[columns.col_value] = ptemplate->get_language();
  row = *(model->append());
  row[columns.col_statistic] = "Source Files";
  row[columns.col_value] = get_number_of_files(filenames);
  row = *(model->append());
  row[columns.col_statistic] = "Lines of Code";
  row[columns.col_value] = get_lines_of_code(filenames, project->get_location());
  
  treeview->set_model(model);
  treeview->append_column("Statistic", columns.col_statistic);
  treeview->append_column("Value", columns.col_value);
  dialog->set_default(*close);
}

Gtk::Dialog* ProjectInfoDialog::get_dialog()
{ return dialog;
}

// Initialize all of the widgest for the project information dialog. This
// function receives the project settings object and sets its information to
// the labels.
void ProjectInfoDialog::initialize_objects ()
{
  Glib::ustring gladefile = string(INSTALL_PREFIX) + "/share/openldev/glade/projectinfo.glade";
  Glib::RefPtr<Gnome::Glade::Xml> xml = Gnome::Glade::Xml::create(gladefile);
  
  xml->get_widget("dialog", dialog);
  xml->get_widget("treeview", treeview);
  xml->get_widget("button_close", close);
}

// This destructor goes through all of the initialized widgets
// and deletes them to free up memory on close of the dialog
ProjectInfoDialog::~ProjectInfoDialog()
{
  delete treeview;
  delete close;
}

// Get the number of files in the project. This is done by counting the
// number of backslashes in the files list
Glib::ustring ProjectInfoDialog::get_number_of_files (Glib::ustring files)
{
  // Count by how many '/' characters are in the string
  int location = files.find("/", 0), count = 0;
  while (location != Glib::ustring::npos)
  {
    count++;
    location = files.find("/", location+1);
  }
  
  // Convert integer properties to character arrays
  gchar buff[5];
  snprintf(buff, sizeof(buff), "%d", count);
  return Glib::ustring(buff);
}

// Get the number of lines of code in the project. The function receives the
// list of filenames and the project source directory location. It then opens
// each of the files & counts how many lines are in the file.
Glib::ustring ProjectInfoDialog::get_lines_of_code(Glib::ustring filenames, Glib::ustring location)
{
  // Loop through to get all of the files
  int loc = filenames.find("/",0), count = 0;
  while (loc != Glib::ustring::npos) 
  {
    // Create filename & then delete that file from the string
    Glib::ustring fn = location + Glib::ustring("src/") + filenames.substr(0, loc);
    filenames.erase(0, loc+1);
    loc = filenames.find("/",0);
    
    // Get the number of lines in the file
    Glib::ustring text = read_file(fn);
    int loc2 = text.find("\n", 0);
    while (loc2 != Glib::ustring::npos)
    {
      count++;
      loc2 = text.find("\n", loc2+1);
    }
    count++;
  }
  
  // Convert integer properties to character arrays
  gchar buff[5];
  snprintf(buff, sizeof(buff), "%d", count);
  return Glib::ustring(buff);
}
